/*-
 * Copyright 2010 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
#ifndef __CX88_VIDEO_H__
#define __CX88_VIDEO_H__

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/conf.h>
#include <sys/lock.h>
#include <sys/sx.h>
#include <sys/module.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <sys/selinfo.h>
#include <sys/poll.h>
#include <sys/fcntl.h>
#include <sys/limits.h>
#include "cx88_common.h"
#include "cx88_if.h"
#include "cx88_i2c.h"
#include "cx88_regs.h"
#include "cx88_video_regs.h"
#include "cx88_video_ioctl.h"
#include "cx88_video_if.h"

#define CX88_VIDEO_MINVER 3
#define CX88_VIDEO_MODVER 3
#define CX88_VIDEO_MAXVER 3

typedef enum
{
   CX88_VIDEO_STREAM_CX23880 = 0,
   CX88_VIDEO_STREAM_CX23885_VIDA,
   CX88_VIDEO_STREAM_CX23885_VIDB,
   CX88_VIDEO_NUM_STREAMS
} cx88_video_stream_t;

typedef struct
{
   cx88_video_stream_t stream;
   cx88_channel_t dma_channels[CX88_VIDEO_CHANNELS];
   uint32_t reg_gp_count[CX88_VIDEO_CHANNELS];
   uint32_t reg_dma_control;
   uint32_t dma_control_mask[CX88_VIDEO_CHANNELS];
   uint32_t reg_pci_intr_mask;
   uint32_t pci_intr_mask;
   uint32_t reg_intr_mask;
   uint32_t reg_intr_stat;
   uint32_t reg_intr_mstat;
   uint32_t risci1_mask[CX88_VIDEO_CHANNELS];
   uint32_t risci2_mask[CX88_VIDEO_CHANNELS];
   uint32_t sync_err_mask[CX88_VIDEO_CHANNELS];
   uint32_t overflow_mask[CX88_VIDEO_CHANNELS];
   uint32_t opcode_err_mask[CX88_VIDEO_CHANNELS];
   uint32_t bus_err_mask;
   uint32_t reg_devcntl2;
   uint32_t pixformats;
} cx88_video_stream_desc;

/* Driver states */
typedef enum
{
   CX88_VIDEO_INVALID = 0, /* Uninitialized */
   CX88_VIDEO_UNCONFIGURED, /* Not configured or formatted */
   CX88_VIDEO_FORMATTED, /* Formatted but not configured */
   CX88_VIDEO_CONFIGURED, /* Configured (I2C devices, buffers, etc.) and formatted */
   CX88_VIDEO_CAPTURING /* Configured, formatted, and capturing */
} cx88_video_state_t;

typedef struct
{
   uint16_t x_header;
   uint16_t x_trailer;
   void *clip_patterns;
   uint32_t clip_pattern_size;
   uint8_t last_sync_buffer;
   volatile uint8_t even_frame_available;
   volatile uint8_t odd_frame_available;
   boolean_t enabled;
} cx88_video_channel_config;

typedef struct
{
   const cx88_video_stream_desc *stream;
   cx88_common_softc *common;
   cx88_i2c_softc* (*i2c_accessor)(cx88_common_softc*, uint8_t);
   device_t videocore_dev;
   cx88_channel channels[CX88_VIDEO_CHANNELS];
   cx88_video_channel_config channel_configs[CX88_VIDEO_CHANNELS];
   struct cdev *devnode;
   int unit; /* Unit number for devnode */
   struct mtx isrlock;
   uint8_t channel_mask;
   uint8_t channels_avail;
   struct sx sxlock;
   struct selinfo selectors;
   boolean_t in_use;
   boolean_t reconfig_requested;
   cx88_video_state_t state;
   cx88_video_format format;
   uint8_t num_frames;
   boolean_t dma_program_valid;
   uint32_t intr_mask;
   volatile uint8_t isr_status; /* Error status from interrupt service routine */
} cx88_video_softc;

int cx88_video_setup(
   cx88_video_softc *sc,
   cx88_video_stream_t stream,
   cx88_common_softc *common,
   cx88_i2c_softc* (*i2c_accessor)(cx88_common_softc*, uint8_t),
   device_t videocore_dev);
void cx88_video_teardown(cx88_video_softc *sc);
int cx88_video_filter(cx88_video_softc *sc);
void cx88_video_intr(cx88_video_softc *sc);
int cx88_video_suspend(cx88_video_softc *sc);
void cx88_video_reset(cx88_video_softc *sc, cx88_video_state_t state);
int cx88_video_request_reconfig(cx88_video_softc *sc);

#endif

